@@ -4,6 +4,12 @@
 
 import gtk
 import cairo
+try:
+    import fast
+except ImportError:
+    HAVE_FAST = False
+else:
+    HAVE_FAST = True
 
 # -- Base classes for drawing sound visualization.
 #
@@ -102,11 +108,26 @@ class GraphView(LayeredGraphView):
 #
 
 class Layer(object):
-    """The base class for layers.
+    """Base class for layers."""
 
-    Implements the cairo surface caching.
+    def __init__(self, layered):
+        self._layered = layered
 
-    """
+    def stack(self, context, width, height):
+        """Paint the layer on top of the passed context."""
+        context.set_operator(cairo.OPERATOR_OVER)
+        self.draw(context, width, height)
+
+    def update(self):
+        self._layered.redraw()
+
+    def draw(context, width, height):
+        raise NotImplemented
+
+
+class CachedLayer(Layer):
+    """Implements surface caching."""
+    
     def __init__(self, layered):
         self._layered = layered
         self._must_draw = True
@@ -119,23 +140,20 @@ class Layer(object):
 
     def update(self):
         self._must_draw = True
-        self._layered.redraw()
+        Layer.update(self)
 
     def stack(self, context, width, height):
-        """ 
-        Paint the layer on top of the passed context.
-        """
         if self._surface is None:
             surface = context.get_target()
             self._surface = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
                                                    width, height)
-        if self._must_draw:
-            # clear context
-            c = cairo.Context(self._surface)
-            c.set_source_rgba(0, 0, 0, 0)
-            c.set_operator(cairo.OPERATOR_SOURCE)
-            c.paint()
 
+        if self._must_draw:
+            # clear the cached surface
+            c = cairo.Context(self._surface)
+            c.set_operator(cairo.OPERATOR_CLEAR)
+            c.paint()
+            c.set_operator(cairo.OPERATOR_OVER)
             self.draw(c, width, height)
             self._must_draw = False
 
@@ -143,18 +161,15 @@ class Layer(object):
         context.set_operator(cairo.OPERATOR_OVER)
         context.paint()
     
-    def draw(context, width, height):
-        raise NotImplemented
 
-
-class WaveformLayer(Layer):
+class WaveformLayer(CachedLayer):
     """A layer for LayeredGraphView.
 
     It paints the graph (the waveform).
 
     """
     def __init__(self, layered, graph):
-        Layer.__init__(self, layered)
+        CachedLayer.__init__(self, layered)
         self._graph = graph
         self.wavecolor = 0.0, 0.47058823529411764, 1.0
         graph.changed.connect(self.update)
@@ -169,10 +184,9 @@ class WaveformLayer(Layer):
 
         # Waveform
         context.set_source_rgb(*self.wavecolor)
-        for i, (mini, maxi) in enumerate(values):
+        for x, (mini, maxi) in enumerate(values):
             # -1 <= mini <= maxi <= 1
             # ystart <= ymin <= ymax <= ystart + height - 1
-            x = i
             ymin = ystart + round((-mini * 0.5 + 0.5) * (height - 1))
             ymax = ystart + round((-maxi * 0.5 + 0.5) * (height - 1))
             if ymin == ymax:
@@ -184,7 +198,10 @@ class WaveformLayer(Layer):
                 context.move_to(x + 0.5, ymin)
                 context.line_to(x + 0.5, ymax)
                 context.stroke()
-        
+
+    if HAVE_FAST:
+        draw_channel = fast.draw_channel
+
     def draw(self, context, width, height):
         channels = self._graph.channels()
         numchan = len(channels)
@@ -234,6 +251,7 @@ class SelectionLayer(Layer):
             context.rectangle(end, 0, width - end, height)
             context.fill()
 
+
 class CursorLayer(Layer):
 
     def __init__(self, layered, cursor):
